// (C) Copyright Andrew Bazan, 3/2000
// Feel free to make modifications or use this code in something else,
// just please give me credit where credit is due. Thanks.

#include "BeAlk3.h"

AlkWindow::AlkWindow()
	: BWindow(BRect(100,100,349,339), "BeAlk3",
		B_DOCUMENT_WINDOW, B_OUTLINE_RESIZE)
{
	BRect r(Bounds());
	
	menubar = new BMenuBar(r, "menu_bar");
	AddChild(menubar);
	
	BMenu *menu;
	menu = new BMenu("File");
	menu->AddItem(new BMenuItem("New fractal", new BMessage(FILE_NEW), 'N'));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("About", new BMessage(FILE_ABOUT), 'A'));
	menu->AddSeparatorItem();
	menu->AddItem(new BMenuItem("Quit", new BMessage(FILE_QUIT), 'Q'));
	menubar->AddItem(menu);
	menu = new BMenu("Fractal");
	lastMarkedFrac = new BMenuItem("Mandelbrot", new BMessage(FRAC_MANDEL));
	lastMarkedFrac->SetMarked(true);
	menu->AddItem(lastMarkedFrac);
	menu->AddItem(new BMenuItem("Julia", new BMessage(FRAC_JULIA)));
	menubar->AddItem(menu);
	menu = new BMenu("Iterations");
	menu->AddItem(new BMenuItem("16", new BMessage(IT_16)));
	lastMarkedIt = new BMenuItem("32", new BMessage(IT_32));
	lastMarkedIt->SetMarked(true);
	menu->AddItem(lastMarkedIt);
	menu->AddItem(new BMenuItem("64", new BMessage(IT_64)));
	menu->AddItem(new BMenuItem("128", new BMessage(IT_128)));
	menu->AddItem(new BMenuItem("256", new BMessage(IT_256)));
	menu->AddItem(new BMenuItem("512", new BMessage(IT_512)));
	menu->AddItem(new BMenuItem("1024", new BMessage(IT_1024)));
	menubar->AddItem(menu);
	menu = new BMenu("Palette");
	lastMarkedPal = new BMenuItem("Mixed 1", new BMessage(PAL_1));
	lastMarkedPal->SetMarked(true);
	menu->AddItem(lastMarkedPal);
	menu->AddItem(new BMenuItem("Mixed 2", new BMessage(PAL_2)));
	menu->AddItem(new BMenuItem("Mixed 3", new BMessage(PAL_3)));
	menu->AddItem(new BMenuItem("All reds", new BMessage(PAL_4)));
	menu->AddItem(new BMenuItem("All greens", new BMessage(PAL_5)));
	menu->AddItem(new BMenuItem("All blues", new BMessage(PAL_6)));
	menubar->AddItem(menu);
	
	r.top = 20;
	
	myView = new AlkView(r);
	AddChild(myView);
	
	Show();
}

AlkWindow::~AlkWindow()
{
}

bool AlkWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

void AlkWindow::MessageReceived(BMessage *message)
{
	switch(message->what)
	{
		case FILE_NEW:
			static_cast<AlkView *>(myView)->NewFractal();
			break;
		case FILE_ABOUT:
		{
			char aDialog[100];
			sprintf(aDialog,
				"BeAlk3 v%1.2f: Copyright 2000 by Andrew Bazan.", VERSION);
			BAlert *aboutAlert = new BAlert("About", aDialog,
				"Cool", NULL, NULL, B_WIDTH_AS_USUAL, B_INFO_ALERT);
			aboutAlert->Go();
			break;
		}
		case FILE_QUIT:
			be_app->PostMessage(B_QUIT_REQUESTED);
			break;
		case FRAC_MANDEL:
			HandleFractal(message, 1);
			break;
		case FRAC_JULIA:
			HandleFractal(message, 2);
			break;
		case IT_16:
			HandleIterations(message, 16);
			break;
		case IT_32:
			HandleIterations(message, 32);
			break;
		case IT_64:
			HandleIterations(message, 64);
			break;
		case IT_128:
			HandleIterations(message, 128);
			break;
		case IT_256:
			HandleIterations(message, 256);
			break;
		case IT_512:
			HandleIterations(message, 512);
			break;
		case IT_1024:
			HandleIterations(message, 1024);
			break;
		case PAL_1:
			HandlePalette(message, 1);
			break;
		case PAL_2:
			HandlePalette(message, 2);
			break;
		case PAL_3:
			HandlePalette(message, 3);
			break;
		case PAL_4:
			HandlePalette(message, 4);
			break;
		case PAL_5:
			HandlePalette(message, 5);
			break;
		case PAL_6:
			HandlePalette(message, 6);
			break;
		default:
			BWindow::MessageReceived(message);
			break;
	}
}

void AlkWindow::HandleIterations(BMessage *message, int iterations)
{
	void *snagSource;
	message->FindPointer("source", &snagSource);
	BMenuItem *sourceItem = static_cast<BMenuItem *>(snagSource);
	static_cast<AlkView *>(myView)->SetIterations(iterations);
	sourceItem->SetMarked(true);
	lastMarkedIt->SetMarked(false);
	lastMarkedIt = sourceItem;
}

void AlkWindow::HandlePalette(BMessage *message, int8 palette)
{
	void *snagSource;
	message->FindPointer("source", &snagSource);
	BMenuItem *sourceItem = static_cast<BMenuItem *>(snagSource);
	static_cast<AlkView *>(myView)->SetPalette(palette);
	sourceItem->SetMarked(true);
	lastMarkedPal->SetMarked(false);
	lastMarkedPal = sourceItem;
}

void AlkWindow::HandleFractal(BMessage *message, int8 fractal)
{
	void *snagSource;
	message->FindPointer("source", &snagSource);
	BMenuItem *sourceItem = static_cast<BMenuItem *>(snagSource);
	static_cast<AlkView *>(myView)->SetFractal(fractal);
	sourceItem->SetMarked(true);
	lastMarkedFrac->SetMarked(false);
	lastMarkedFrac = sourceItem;
}