// (C) Copyright Andrew Bazan, 3/2000
// Feel free to make modifications or use this code in something else,
// just please give me credit where credit is due. Thanks.

#ifndef BEALK3_H
#define BEALK3_H

#include <Application.h>
#include <Window.h>
#include <View.h>
#include <Menu.h>
#include <MenuBar.h>
#include <MenuItem.h>
#include <Alert.h>
#include <Bitmap.h>
#include <stdio.h>
#include <math.h>

const float VERSION	= 1.0;

const uint32 FILE_NEW		= 'FNew';
const uint32 FILE_ABOUT		= 'FAbt';
const uint32 FILE_QUIT		= 'FQui';

const uint32 IT_16			= 'IT1';
const uint32 IT_32			= 'IT2';
const uint32 IT_64			= 'IT3';
const uint32 IT_128			= 'IT4';
const uint32 IT_256			= 'IT5';
const uint32 IT_512			= 'IT6';
const uint32 IT_1024		= 'IT7';

const uint32 PAL_1			= 'Pal1';
const uint32 PAL_2			= 'Pal2';
const uint32 PAL_3			= 'Pal3';
const uint32 PAL_4			= 'Pal4';
const uint32 PAL_5			= 'Pal5';
const uint32 PAL_6			= 'Pal6';

const uint32 FRAC_MANDEL	= 'FMan';
const uint32 FRAC_JULIA		= 'FJul';

class FastComplex
{
	public:
		FastComplex();
		FastComplex(const double &r, const double &i);
		FastComplex Add(const FastComplex &rhs);
		FastComplex Square();
		double AbsoluteValue();
		double GetReal() const;
		double GetImag() const;
	
	private:
		double rPart;
		double iPart;
};

class AlkApp : public BApplication
{
	public:
		AlkApp();
		virtual ~AlkApp();
};

class AlkWindow : public BWindow
{
	public:
		AlkWindow();
		virtual ~AlkWindow();
		virtual bool QuitRequested();
		virtual void MessageReceived(BMessage *message);

	private:
		void HandleIterations(BMessage *message, int iterations);
		void HandlePalette(BMessage *message, int8 palette);
		void HandleFractal(BMessage *message, int8 fractal);

		BView *myView;
		BMenuBar *menubar;
		BMenuItem *lastMarkedFrac, *lastMarkedIt, *lastMarkedPal;
};

class AlkView : public BView
{
	public:
		AlkView(BRect frame);
		virtual ~AlkView();
		void NewFractal();
		void SetIterations(int iterations);
		void SetPalette(int8 palette);
		void SetFractal(int8 fractal);
		virtual void Draw(BRect updateRect);
		virtual void MouseDown(BPoint myPoint);
		virtual void MouseUp(BPoint myPoint);
		virtual void FrameResized(float width, float height);
		
	private:
		void Fractalize();
		int16 Iterate(const FastComplex &c);
		int16 IterateJulia(FastComplex z);
		void GetColor(const int16 &iterations, int16 &r, int16 &g, int16 &b);
		
		int frameWidth, frameHeight, oldWidth, oldHeight;
		BBitmap *myFB;
		bool legitDown, running;
		int downX, downY, maxIterations;
		double initReal, initImag;
		double curVStep, curHStep;
		int8 paletteType, fractalType;
};

#endif